
# THIS FILE IS GENERATED FROM wofry SETUP.PY
short_version = '1.0.13'
version = '1.0.13'
full_version = '1.0.13'
git_revision = '3cc53b6a1b4f4b6aab5aae947bc48a93c0bffb0c'
release = True

if not release:
    version = full_version
    short_version += ".dev"
